// This file was automatically generated by genny.
// Any changes will be lost if this file is regenerated.
// see https://github.com/cheekybits/genny

/*
 * Cadence - The resource-oriented smart contract programming language
 *
 * Copyright 2019-2021 Dapper Labs, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Based on https://github.com/wk8/go-ordered-map, Copyright Jean Rougé
 *
 */

package orderedmap

import "container/list"

// StringStringOrderedMap
//
type StringStringOrderedMap struct {
	pairs map[string]*StringStringPair
	list  *list.List
}

// NewStringStringOrderedMap creates a new StringStringOrderedMap.
func NewStringStringOrderedMap() *StringStringOrderedMap {
	return &StringStringOrderedMap{
		pairs: make(map[string]*StringStringPair),
		list:  list.New(),
	}
}

// Clear removes all entries from this ordered map.
func (om *StringStringOrderedMap) Clear() {
	om.list.Init()
	// NOTE: Range over map is safe, as it is only used to delete entries
	for key := range om.pairs { //nolint:maprangecheck
		delete(om.pairs, key)
	}
}

// Get returns the value associated with the given key.
// Returns nil if not found.
// The second return value indicates if the key is present in the map.
func (om *StringStringOrderedMap) Get(key string) (result string, present bool) {
	var pair *StringStringPair
	if pair, present = om.pairs[key]; present {
		return pair.Value, present
	}
	return
}

// GetPair returns the key-value pair associated with the given key.
// Returns nil if not found.
func (om *StringStringOrderedMap) GetPair(key string) *StringStringPair {
	return om.pairs[key]
}

// Set sets the key-value pair, and returns what `Get` would have returned
// on that key prior to the call to `Set`.
func (om *StringStringOrderedMap) Set(key string, value string) (oldValue string, present bool) {
	var pair *StringStringPair
	if pair, present = om.pairs[key]; present {
		oldValue = pair.Value
		pair.Value = value
		return
	}

	pair = &StringStringPair{
		Key:   key,
		Value: value,
	}
	pair.element = om.list.PushBack(pair)
	om.pairs[key] = pair

	return
}

// Delete removes the key-value pair, and returns what `Get` would have returned
// on that key prior to the call to `Delete`.
func (om *StringStringOrderedMap) Delete(key string) (oldValue string, present bool) {
	var pair *StringStringPair
	pair, present = om.pairs[key]
	if !present {
		return
	}

	om.list.Remove(pair.element)
	delete(om.pairs, key)
	oldValue = pair.Value

	return
}

// Len returns the length of the ordered map.
func (om *StringStringOrderedMap) Len() int {
	return len(om.pairs)
}

// Oldest returns a pointer to the oldest pair.
func (om *StringStringOrderedMap) Oldest() *StringStringPair {
	return listElementToStringStringPair(om.list.Front())
}

// Newest returns a pointer to the newest pair.
func (om *StringStringOrderedMap) Newest() *StringStringPair {
	return listElementToStringStringPair(om.list.Back())
}

// Foreach iterates over the entries of the map in the insertion order, and invokes
// the provided function for each key-value pair.
func (om *StringStringOrderedMap) Foreach(f func(key string, value string)) {
	for pair := om.Oldest(); pair != nil; pair = pair.Next() {
		f(pair.Key, pair.Value)
	}
}

// StringStringPair
//
type StringStringPair struct {
	Key   string
	Value string

	element *list.Element
}

// Next returns a pointer to the next pair.
func (p *StringStringPair) Next() *StringStringPair {
	return listElementToStringStringPair(p.element.Next())
}

// Prev returns a pointer to the previous pair.
func (p *StringStringPair) Prev() *StringStringPair {
	return listElementToStringStringPair(p.element.Prev())
}

func listElementToStringStringPair(element *list.Element) *StringStringPair {
	if element == nil {
		return nil
	}
	return element.Value.(*StringStringPair)
}
